'use strict';

module.exports.create = function (cli, dnsd) {
  function runUdp() {
    var server = require('dgram').createSocket({
      type: cli.udp6 ? 'udp6' : 'udp4'
    , reuseAddr: true
    });
    server.bind({
      port: cli.port
    , address: cli.address
    });

    var handlers = {};
    handlers.onError = function (err) {
      if ('EACCES' === err.code) {
        console.error("");
        console.error("EACCES: Couldn't bind to port. You probably need to use sudo, authbind, or setcap.");
        console.error("");
        process.exit(123);
        return;
      }
      console.error("UDP Server Error:");
      console.error(err.stack);
      server.close(function () {
        setTimeout(runUdp, 1000);
      });
      //throw new Error(err);
    };

    handlers.onMessage = function (nb, rinfo) {
      //console.log('[DEBUG] got a UDP message', nb.length);
      //console.log(nb.toString('hex'));

      dnsd.onMessage(nb, function (err, newAb, dbgmsg, unicastResponse) {
        // TODO send legit error message
        const address = cli.mdns && !unicastResponse ? '224.0.0.251' : rinfo.address;
        if (err) { server.send(Buffer.from([0x00]), rinfo.port, address); return; }
        server.send(newAb, rinfo.port, address, function () {
          console.log('[dnsd.onMessage] ' + dbgmsg, rinfo.port, address);
        });
      });
    };

    handlers.onListening = function () {
      /*jshint validthis:true*/
      var server = this;

      if (cli.mdns || '224.0.0.251' === cli.nameserver) {
        server.addMembership(cli.nameserver);
      }

      console.log('');
      console.log('Bound and Listening:');
      console.log(server.address().address + '#' + server.address().port + ' (' + server.type + ')');

      if (cli.mdns) {
        // special startup case to advertise local dns-sd records
        handlers.onMessage('_services._dns-sd._udp.local', { address: '224.0.0.251', port: 5353 });
      }
    };

    server.on('error', handlers.onError);
    server.on('message', handlers.onMessage);
    server.on('listening', handlers.onListening);

    return server;
  }

  return runUdp();
};
